<?php
/**
 * 2007-2018 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2018 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

namespace KiwizInvoiceHandler\Service;

use GuzzleHttp\Client;
use Exception;
use AddressCore as Address;
use CountryCore as Country;
use CustomerCore as Customer;
use CarrierCore as Carrier;
use PDFCore as PDF;
use ToolsCore as Tools;
use OrderCore as Order;
use OrderSlipCore as OrderSlip;
use ManufacturerCore as Manufacturer;
use ContextCore as Context;
use ApiCredential;
use ApiError;

class KiwizApi extends Client
{
    /**
     * @var string
     */
    private $baseUri = 'https://api.kiwiz.io/';
    /**
     * @var array
     */
    private $entities = array('invoice', 'creditmemo');

    private $apiCredential;

    private $smarty;

    private $maxFileSize = 1024;

    private $context;

    /**
     * KiwizApi constructor.
     * @param  $smarty
     * @param array $config
     */
    public function __construct($smarty = null, $config = array())
    {
        parent::__construct($config);
        $this->apiCredential = $this->getApiCredentialFromDb();
        $this->smarty = $smarty;
        $this->context = Context::getContext();
    }

    private function getApiCredentialFromDb()
    {
        $apiCredential = new ApiCredential();

        return $apiCredential->getByIdShop();
    }

    public function getApiCredential()
    {
        return $this->apiCredential;
    }

    /**
     * @return null
     */
    public function getToken()
    {

        return $this->token;
    }

    /**
     * @param $token
     */
    public function setToken($token)
    {
        $this->token = $token;
    }


    /**
     * @param $username
     * @param $password
     * @param $subscriptionId
     * @param null $notification_emails
     * @return Exception
     * @throws \PrestaShopException
     */
    public function generateToken($username, $password, $subscriptionId, $notification_emails = null)
    {
        $this->apiCredential->username = $username;
        $this->apiCredential->password = $password;
        $this->apiCredential->subscription_id = $subscriptionId;
        $this->apiCredential->notification_emails = $notification_emails;

        $result = $this->getAuth();

        if (!is_object($result)) {
            $this->apiCredential->token = $result;
            $this->apiCredential->date_launch = date("Y-m-d H:i:s");
            $this->apiCredential->save();
        } else {
            return $result;
        }

        return $this->apiCredential->token;
    }

    /**
     * @return Exception or string
     */
    private function getAuth()
    {
        $httpMethod = "POST";
        $uri = $this->baseUri . 'token/generate';
        $form_params =
            array(
                'username' => $this->apiCredential->username,
                'password' => $this->apiCredential->password,
                'subscription_id' => $this->apiCredential->subscription_id
            );

        $client = new Client();
        $headers = array(
            'Content-Type' => 'application/x-www-form-urlencoded',
        );

        try {
            $response = $client->request(
                $httpMethod,
                $uri,
                array(
                    'headers' => $headers,
                    'form_params' => $form_params
                )
            );
        } catch (Exception $e) {
            return $e;
        }
        $result = json_decode($response->getBody()->getContents());

        return $result->token;
    }

    public function save($entity, $object)
    {
        $this->checkToken();

        $key = array_search($entity, $this->entities);
        if (!is_int($key)) {
            $this->vd('false entity');
            die;
        }
        $method = 'setData' . Tools::ucfirst($entity);
        $data = $this->$method($object);

        $document = $this->generatePdf($entity, $object);

        if (!file_exists($document)) {
            $this->vd('File does not exist');
            die;
        }

        $httpMethod = "POST";
        $uri = $this->baseUri . $entity . '/save';

        $client = new Client();
        $headers = array(
            'Authorization' => $this->apiCredential->token,
            'accept' => 'application/json',
        );

        $multipart = array(
            array(
                'name' => 'document',
                'contents' => fopen($document, 'r'),
                'filename' => 'document.pdf',
                'Content-Type' => 'application/pdf'
            ),
            array(
                'name' => 'data',
                'contents' => json_encode($data),
            )
        );

        try {
            $response = $client->request(
                $httpMethod,
                $uri,
                array(
                    'headers' => $headers,
                    'multipart' => $multipart,
                )
            );
        } catch (\Exception $e) {
            if ($e->getCode() == 401) {
                $tryRegenerate = $this->reGenerateToken();
                if (get_class($tryRegenerate) == 'GuzzleHttp\Exception\ClientException') {
                    return $tryRegenerate;
                }

                return $this->save($entity, $object);
            }

            return $e;
        }

        return json_decode(trim($response->getBody()->getContents(), '"'));
    }

    /**
     *
     */
    private function checkToken()
    {
        if (is_null($this->apiCredential->token) || !$this->apiCredential->token) {
            $this->vd('no token');
            die;
        }
    }

    /**
     * @param $var
     */
    public function vd($var)
    {
        echo '<pre style="color:#fff;background: #211f1f;padding: 10px;">';
        var_dump($var);
        echo '</pre>';
    }

    public function generatePdf($entity, $object, $ps = false)
    {
        if ($entity == 'creditmemo') {
            $entity = 'orderSlip';
        }

        $pdf = new PDF($object, Tools::ucfirst($entity), $this->context->smarty);

        if ($ps) {
            return $pdf->render(false);
        }

        file_put_contents(_PS_ROOT_DIR_ . '/upload/document.pdf', $pdf->render(false));
        $path = _PS_ROOT_DIR_ . '/upload/document.pdf';
        if ($this->maxFileSize > filesize($path)) {
            $this->vd('file is too big max file value is ' . $this->maxFileSize);

            return false;
        }

        return $path;
    }

    /**
     */
    public function reGenerateToken()
    {
        $result = $this->getAuth();
//        $this->vd($result);die;
        if (is_object($result)) {
//            $this->vd('Regenerate errors : ' . $result->getMessage());

            $apiError = new ApiError();
            $apiError->setIdOrderFile('0');
            $apiError->setFileType('--');
            $apiError->setCodeError($result->getCode());
            $apiError->setMessage('Regenerate errors : ' . $result->getMessage());
            $apiError->save();

            return $result;
        }

        $this->apiCredential->token = $result;
//        $this->apiCredential->save();

        return true;
    }

    /**
     * @param \Psr\Http\Message\UriInterface|string $entity
     * @param array $parameter
     * @return \Psr\Http\Message\ResponseInterface|string
     */
    public function get($entity, $parameter)
    {
        $this->checkToken();

        $key = array_search($entity, $this->entities);
        if (!is_int($key)) {
            $this->vd('false entity');

            return false;
        }

        $httpMethod = "GET";
        $uri = $this->baseUri . $entity . '/get?' . $parameter['name'] . '=' . $parameter['value'];

        $client = new Client();
        $headers = array(
            'Authorization' => $this->apiCredential->token,
            'Content-Type' => 'application/x-www-form-urlencoded',
        );

        try {
            $response = $client->request(
                $httpMethod,
                $uri,
                array(
                    'headers' => $headers,
                )
            );
        } catch (Exception $e) {
            if ($e->getCode() == 401) {
                $tryRegenerate = $this->reGenerateToken();
                if (get_class($tryRegenerate) == 'GuzzleHttp\Exception\ClientException') {
                    return $tryRegenerate;
                }

                return $this->get($entity, $parameter);
            }

            return $e;
        }

        return trim($response->getBody()->getContents(), '"');
    }

    /**
     * @return bool|string
     */
    public function getQuota()
    {
        $this->checkToken();

        $httpMethod = "GET";
        $uri = $this->baseUri . 'quota/info';

        $client = new Client();
        $headers = array(
            'Authorization' => $this->apiCredential->token,
            'Content-Type' => 'application/json',
        );
        try {
            $response = $client->request(
                $httpMethod,
                $uri,
                array(
                    'headers' => $headers,
                )
            );
        } catch (Exception $e) {
            if ($e->getCode() == 401) {
                $tryRegenerate = $this->reGenerateToken();
//                $this->vd($tryRegenerate);die;
                if (get_class($tryRegenerate) == 'GuzzleHttp\Exception\ClientException') {
                    return $tryRegenerate;
                }

                return $this->getQuota();
            }

            return $e;
        }

        return json_decode(trim($response->getBody()->getContents(), '"'));
    }

    /**
     * @param $invoice
     * @return array
     */
    public function setDataInvoice($invoice)
    {
        $items = array();
        if (count($invoice->getProducts()) > 0) {
            $items = $this->setDataItems($invoice->getProducts());
        } elseif (count($invoice->getProductList()) > 0) {
            $items = $this->setDataItems($invoice->getProductList());
        }
        $order = $invoice->getOrder();

        $idAddressBilling = $order->id_address_invoice;
        $idAddressShipping = $order->id_address_delivery;

        $addressBilling = new Address($idAddressBilling);
        $countryBilling = new Country($addressBilling->id_country);
        $countryBillingCode = $countryBilling->iso_code;

        if ($idAddressShipping != $idAddressShipping) {
            $addressShipping = new Address($idAddressShipping);
            $countryShipping = new Country($addressShipping->id_country);
            $countryShippingCode = $countryShipping->iso_code;
        } else {
            $addressShipping = $addressBilling;
            $countryShippingCode = $countryBillingCode;
        }

        $customer = $invoice->getOrder()->getCustomer();

        $shippingMethod = new Carrier($order->id_carrier);
        $shippingTaxAmount = (float)$invoice->total_shipping_tax_incl - (float)$invoice->total_shipping_tax_excl;
        $grandTotalTaxAmount = (float)$invoice->total_paid_tax_incl - (float)$invoice->total_paid_tax_excl;
        $grandTotalTaxAmount = (float)number_format($grandTotalTaxAmount, 2);

        return array(
            'increment_id' => $invoice->getInvoiceNumberFormatted($this->context->language->id),
            'date' => gmdate('Y-m-d H:i:s', strtotime($invoice->date_add)),
            'email' => $customer->email,
            'billing_address' =>
                array(
                    'firstname' => $addressBilling->firstname,
                    'lastname' => $addressBilling->lastname,
                    'company' => $addressShipping->company,
                    'street' => $addressBilling->address1,
                    'postcode' => $addressBilling->postcode,
                    'city' => $addressBilling->city,
                    'country_code' => $countryBillingCode
                ),
            'payment_method' => $order->payment,
            'shipping_address' =>
                array(
                    'firstname' => $addressShipping->firstname,
                    'lastname' => $addressShipping->lastname,
                    'company' => $addressShipping->company,
                    'street' => $addressShipping->address1,
                    'postcode' => $addressShipping->postcode,
                    'city' => $addressShipping->city,
                    'country_code' => $countryShippingCode
                ),
            'shipping_method' => $shippingMethod->name,
            'shipping_amount_excl_tax' => $invoice->total_shipping_tax_excl,
            'shipping_tax_amount' => (string)$shippingTaxAmount,
            'items' => $items,
            'grand_total_excl_tax' => $invoice->total_paid_tax_excl,
            'grand_total_tax_amount' => (string)$grandTotalTaxAmount,
        );
    }

    /**
     * @param $items
     * @return array
     */
    private function setDataItems($items)
    {
        $itemArray = array();
        foreach ($items as $item) {
            $manufacturer = new Manufacturer((int)$item['id_manufacturer']);
            $totalTaxAmount = (float)$item['total_price_tax_incl'] - (float)$item['total_price_tax_excl'];
            $totalTaxAmount = (float)number_format($totalTaxAmount, 2);

            $itemArray [] = array(
                'sku' => $item['reference'],
                'ean13' => $item['ean13'],
                'product_name' => $item['product_name'],
                'manufacturer' => $manufacturer->name,
                'qty' => $item['product_quantity'],
                'row_total_excl_tax' => $item['total_price_tax_excl'],
                'row_total_tax_amount' => $totalTaxAmount
            );
        }

        return $itemArray;
    }

    /**
     * @param $creditmemo
     * @return array
     */
    public function setDataCreditmemo($creditmemo)
    {
        $customer = new Customer($creditmemo->id_customer);
        $totalTaxExcl = (float)number_format($creditmemo->total_products_tax_excl, 2);
        $totalTax = (float)number_format($creditmemo->total_products_tax_incl, 2)
            - $totalTaxExcl;
        $totalTax = (float)number_format($totalTax, 2);

//        $apiError = new ApiError();
//        $apiError->setIdOrderFile($creditmemo->id);
//        $apiError->setFileType('slip');
//        $apiError->setCodeError('200');
//        $apiError->setMessage(
//            'grand_total_tax_amount : '
//            . $totalTax
//            . ' grand_total_excl_tax : '
//            . (float)number_format($creditmemo->total_products_tax_incl, 2)
//        );
//        $apiError->save();
//
        return array(
            'increment_id' => $this->getSlipNumberFormated($creditmemo->id),
            'date' => gmdate('Y-m-d H:i:s', strtotime($creditmemo->date_add)),
//            'grand_total_excl_tax' => (string)$creditmemo->total_products_tax_excl,
            'grand_total_excl_tax' => $totalTaxExcl,
            'grand_total_tax_amount' => (string)$totalTax,
            'email' => $customer->email
        );
    }

    private function getSlipNumberFormated($slipId)
    {
        $slipId = (int)$slipId;
        $incrementId = '';

        if ($slipId < 10) {
            $incrementId = '0000' . $slipId;
        } elseif ((int)$slipId >= 10 && (int)$slipId < 99) {
            $incrementId = '000' . (string)$slipId;
        } elseif ((int)$slipId >= 100 && (int)$slipId < 999) {
            $incrementId = '00' . (string)$slipId;
        } elseif ((int)$slipId >= 1000 && (int)$slipId < 9999) {
            $incrementId = '0' . (string)$slipId;
        }

        return $incrementId;
    }
}
